@@ -464,3 +464,55 @@ def ListDockerImages(parent: str, page_size: int, limit: int):
           limit=limit,
       )
   )
+
+
+def CopyRepository(
+    source_repo,
+    dest_repo_name,
+    continue_on_skipped_version=False,
+    max_version_age_days=0,
+    include_all_attachments=False,
+    exclude_all_tags=False,
+):
+  """Copies a repository."""
+  client = GetClient()
+  messages = GetMessages()
+  copy_req = messages.CopyRepositoryRequest(
+      sourceRepository=source_repo,
+      behavior=messages.Behavior(
+          continueOnSkippedVersion=continue_on_skipped_version,
+          maxVersionAgeDays=max_version_age_days,
+          allAttachmentsIncluded=include_all_attachments,
+          allTagsExcluded=exclude_all_tags,
+      ),
+  )
+  req = messages.ArtifactregistryProjectsLocationsRepositoriesCopyRepositoryRequest(
+      destinationRepository=dest_repo_name,
+      copyRepositoryRequest=copy_req,
+  )
+  return client.projects_locations_repositories.CopyRepository(req)
+
+
+def ExportArtifact(version, tag, gcs_destination):
+  """Exports an artifact by version or tag."""
+  client = GetClient()
+  messages = GetMessages()
+  if version:
+    req = messages.ArtifactregistryProjectsLocationsRepositoriesExportArtifactRequest(
+        repository=version.Parent().Parent().RelativeName(),
+        exportArtifactRequest=messages.ExportArtifactRequest(
+            gcsPath=gcs_destination,
+            sourceVersion=version.RelativeName(),
+        ),
+    )
+  elif tag:
+    req = messages.ArtifactregistryProjectsLocationsRepositoriesExportArtifactRequest(
+        repository=tag.Parent().Parent().RelativeName(),
+        exportArtifactRequest=messages.ExportArtifactRequest(
+            gcsPath=gcs_destination,
+            sourceTag=tag.RelativeName(),
+        ),
+    )
+  else:
+    raise ValueError("Either version or tag must be specified.")
+  return client.projects_locations_repositories.ExportArtifact(req)
