@@ -16,11 +16,13 @@
 
 import argparse
 import itertools
+import textwrap
 from typing import Any
 
 import frozendict
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import arg_parsers
+from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope.concepts import concepts
 from googlecloudsdk.calliope.concepts import deps
 from googlecloudsdk.command_lib.backupdr import util
@@ -180,6 +182,18 @@ def GetBackupResourceSpec():
   )
 
 
+def GetDataSourceResourceSpec():
+  return concepts.ResourceSpec(
+      'backupdr.projects.locations.backupVaults.dataSources',
+      resource_name='Data Source',
+      locationsId=LocationAttributeConfig(),
+      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
+      backupVaultsId=BackupVaultAttributeConfig(),
+      dataSourcesId=DataSourceAttributeConfig(),
+      disable_auto_completers=False,
+  )
+
+
 def AddManagementServerResourceArg(parser, help_text):
   """Adds an argument for management server to parser."""
   name = 'management_server'
@@ -256,8 +270,12 @@ def AddCreateBackupPlanAssociationFlags(parser):
 
   AddResourceType(
       parser,
-      """Type of resource to which the backup plan should be applied.
-          E.g., `compute.<UNIVERSE_DOMAIN>.com/Instance` """,
+      textwrap.dedent("""\
+      Type of resource to which the backup plan should be applied.
+
+      For example:
+      * `compute.<UNIVERSE_DOMAIN>/Instance` for Compute Engine instances.
+      * `file.<UNIVERSE_DOMAIN>/Instance` for Filestore instances."""),
   )
 
 
@@ -307,9 +325,10 @@ def AddTriggerBackupFlags(parser):
       ],
   ).AddToParser(parser)
 
-  parser.add_argument(
+  group = parser.add_mutually_exclusive_group()
+  group.add_argument(
       '--backup-rule-id',
-      required=True,
+      required=False,
       type=str,
       help=(
           'Name of an existing backup rule to use for creating an on-demand'
@@ -317,6 +336,22 @@ def AddTriggerBackupFlags(parser):
       ),
   )
 
+  group.add_argument(
+      '--custom-retention-days',
+      required=False,
+      type=int,
+      help=(
+          'Duration for which backup data will be retained.'
+      ),
+  )
+
+  base.Argument(
+      '--labels',
+      metavar='KEY=VALUE',
+      type=arg_parsers.ArgDict(value_type=str),
+      action=arg_parsers.UpdateAction,
+      help='Labels to be applied to the backup.').AddToParser(parser)
+
 
 def AddNetwork(parser, required=False):
   """Adds a positional network argument to parser.
@@ -539,8 +574,8 @@ def AddBackupRetentionInheritance(parser):
 
 
 def AddUpdateBackupFlags(parser):
-  """Adds the --enforced-retention-end-time and --expire-time flags to the given parser."""
-  group = parser.add_argument_group('Update Backup Flags', required=True)
+  """Adds update backup flags to the given parser."""
+  group = parser.add_argument_group('Update Backup Flags')
   help_text = """
    Backups cannot be deleted until this time or later. This period can be extended, but not shortened.
    It should be specified in the format of "YYYY-MM-DD".
@@ -564,6 +599,44 @@ def AddUpdateBackupFlags(parser):
       ),
   )
 
+  # A backup can only be updated with one of these flags at a time.
+  label_update_group = parser.add_mutually_exclusive_group()
+
+  label_update_group.add_argument(
+      '--update-labels',
+      metavar='KEY=VALUE',
+      type=arg_parsers.ArgDict(value_type=str),
+      action=arg_parsers.UpdateAction,
+      help=textwrap.dedent("""\
+      A list of labels to apply to the backup.
+
+      Keys must start with a lowercase letter and contain only lowercase
+      letters, numbers, and hyphens, and must be between 1 and 63 characters
+      long. Values must contain only lowercase letters, numbers, and hyphens,
+      and must be between 0 and 63 characters long.
+
+      For example: `--update-labels=env=prod,team=storage`
+      """))
+
+  label_update_group.add_argument(
+      '--remove-labels',
+      metavar='KEY',
+      type=arg_parsers.ArgList(min_length=1),
+      help="""
+      A list of label keys to remove from the backup.
+
+      If a label does not exist, it is silently ignored.
+      """)
+
+  label_update_group.add_argument(
+      '--clear-labels',
+      action='store_true',
+      help="""
+      Remove all labels from the backup.
+
+      If the backup has no labels, this operation is a no-op.
+      """)
+
 
 def AddOutputFormat(parser, output_format):
   parser.display_info.AddFormat(output_format)
@@ -658,33 +731,33 @@ def AddBackupVaultAccessRestrictionEnumFlag(parser, command: str):
     )
     default = 'within-org'
   else:
-    help_text = """
-Authorize certain sources and destinations for data being sent into, or restored from the current backup vault.
+    help_text = textwrap.dedent("""\
+        Authorize certain sources and destinations for data being sent into, or restored from the current backup vault.
 
-Access restrictions can be modified to be more or less restrictive.
+        Access restrictions can be modified to be more or less restrictive.
 
-    ::: More restrictive access restriction update will fail by default if there will be non compliant Data Sources.
-    To allow such updates, use the --force-update-access-restriction flag.
-    :::  For Google Cloud Console resources, the following changes are allowed to make access restrictions more restrictive:
-        *   `UNRESTRICTED` to `WITHIN_PROJECT` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` / `WITHIN_ORGANIZATION`
-        *   `WITHIN_PROJECT` to `WITHIN_ORGANIZATION` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
+            ::: More restrictive access restriction update will fail by default if there will be non compliant Data Sources.
+            To allow such updates, use the --force-update-access-restriction flag.
+            :::  For Google Cloud Console resources, the following changes are allowed to make access restrictions more restrictive:
+                *   `UNRESTRICTED` to `WITHIN_PROJECT` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` / `WITHIN_ORGANIZATION`
+                *   `WITHIN_PROJECT` to `WITHIN_ORGANIZATION` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
 
-    ::: For Management Server resources, the following changes are allowed to make access restrictions more restrictive:
-        *   `UNRESTRICTED` to `WITHIN_PROJECT` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` / `WITHIN_ORGANIZATION`
-        *   `WITHIN_PROJECT` to `WITHIN_ORGANIZATION` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
+            ::: For Management Server resources, the following changes are allowed to make access restrictions more restrictive:
+                *   `UNRESTRICTED` to `WITHIN_PROJECT` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` / `WITHIN_ORGANIZATION`
+                *   `WITHIN_PROJECT` to `WITHIN_ORGANIZATION` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
 
-    :::   For both Google Cloud Console and Management Server resources, the following changes are allowed to make access restrictions more restrictive:
-        *   `UNRESTRICTED` to `WITHIN_PROJECT` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` / `WITHIN_ORGANIZATION`
-        *   `WITHIN_PROJECT` to `WITHIN_ORGANIZATION` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
+            :::   For both Google Cloud Console and Management Server resources, the following changes are allowed to make access restrictions more restrictive:
+                *   `UNRESTRICTED` to `WITHIN_PROJECT` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` / `WITHIN_ORGANIZATION`
+                *   `WITHIN_PROJECT` to `WITHIN_ORGANIZATION` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
 
-    ::: For Google Cloud Console resources,  the following changes are allowed to make access restrictions less restrictive:
-        *   `WITHIN_ORGANIZATION` to `UNRESTRICTED` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
-        *   `WITHIN_PROJECT` to `UNRESTRICTED`
-        *   `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` to `UNRESTRICTED`
+            ::: For Google Cloud Console resources,  the following changes are allowed to make access restrictions less restrictive:
+                *   `WITHIN_ORGANIZATION` to `UNRESTRICTED` / `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA`
+                *   `WITHIN_PROJECT` to `UNRESTRICTED`
+                *   `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` to `UNRESTRICTED`
 
-    ::: For Management Server resources, the following changes are allowed to make access restrictions less restrictive:
-        *   `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` to `UNRESTRICTED`
-    """
+            ::: For Management Server resources, the following changes are allowed to make access restrictions less restrictive:
+                *   `WITHIN_ORG_BUT_UNRESTRICTED_FOR_BA` to `UNRESTRICTED`
+        """)
     default = None
 
   parser.add_argument(
@@ -709,6 +782,19 @@ def AddForceUpdateAccessRestriction(parser):
   )
 
 
+def AddKmsKey(parser):
+  """Adds the --kms-key flag to the given parser."""
+  parser.add_argument(
+      '--kms-key',
+      required=False,
+      type=str,
+      help=(
+          'The Cloud KMS key resource name to be used for encryption. Format:'
+          ' projects/{project}/locations/{location}/keyRings/{ring}/cryptoKeys/{key}'
+      ),
+  )
+
+
 def AddResourceType(parser, help_text):
   """Adds a positional resource-type argument to parser.
 
@@ -736,13 +822,14 @@ def AddLogRetentionDays(parser, hidden=True):
       required=False,
       hidden=hidden,
       type=int,
-      help=("""Configures how long logs will be stored. It is defined in "days".
+      help=textwrap.dedent("""\
+          Configures how long logs will be stored. It is defined in "days".
           This value should be greater than or equal to minimum enforced log
           retention duration of the backup vault."""),
   )
 
 
-def AddBackupRule(parser, required=True):
+def AddBackupRule(parser, required=False):
   """Adds a positional backup-rule argument to parser.
 
   Args:
@@ -802,13 +889,12 @@ def AddBackupRule(parser, required=True):
       ),
       action='append',
       metavar='PROPERTY=VALUE',
-      help=(
-          f"""Backup rule that defines parameters for when and how a backup
+      help=textwrap.dedent(f"""\
+          Backup rule that defines parameters for when and how a backup
           is created. This flag can be repeated to create more backup rules.
 
           {BACKUP_RULE_COMMON_HELP_TEXT}
-          """
-      ),
+          """),
   )
 
 
@@ -864,16 +950,15 @@ def AddUpdateBackupRule(parser: argparse.ArgumentParser):
       ),
       action='append',
       metavar='PROPERTY=VALUE',
-      help=(
-          f"""Full definition of an existing backup rule with updated values.
+      help=textwrap.dedent(f"""\
+          Full definition of an existing backup rule with updated values.
           The existing backup rule is replaced by this new set of values.
           This flag can be repeated to update multiple backup rules.
           It is not allowed to pass the same rule-id in this flag more than once
           in the same command.
 
           {BACKUP_RULE_COMMON_HELP_TEXT}
-          """
-      ),
+          """),
   )
 
 
@@ -936,10 +1021,9 @@ def AddAddBackupRule(parser):
       ),
       action='append',
       metavar='PROPERTY=VALUE',
-      help=(
-          """Parameters of backup rule to be added to the Backup Plan. This flag can be repeated to add more backup rules.
-          """
-      ),
+      help=textwrap.dedent("""\
+          Parameters of backup rule to be added to the Backup Plan. This flag can be repeated to add more backup rules.
+          """),
   )
 
 
@@ -953,10 +1037,9 @@ def AddRemoveBackupRule(parser):
       '--remove-backup-rule',
       required=False,
       type=str,
-      help=(
-          """Name of an existing backup rule to be removed from the Backup Plan. This flag can be repeated to remove more backup rules.
-          """
-      ),
+      help=textwrap.dedent("""\
+          Name of an existing backup rule to be removed from the Backup Plan. This flag can be repeated to remove more backup rules.
+          """),
       action='append',
       metavar='RULE-ID',
   )
@@ -974,3 +1057,56 @@ def AddBackupRulesFromFile(parser):
       type=arg_parsers.FileContents(),
       help='Path to a YAML or JSON file containing backup rules.',
   )
+
+
+def AddCloudsqlEdition(parser, help_text):
+  """Adds a positional edition argument to parser.
+
+  Args:
+    parser: argparse.Parser: Parser object for command line inputs.
+    help_text: Help text for the edition argument.
+  """
+  parser.add_argument(
+      '--cloudsql-edition',
+      required=False,
+      type=str,
+      help=help_text,
+  )
+
+
+def AddMaxCustomOnDemandRetentionDays(parser):
+  """Adds a max-custom-on-demand-retention-days argument to parser."""
+  parser.add_argument(
+      '--max-custom-on-demand-retention-days',
+      required=False,
+      type=int,
+      help=textwrap.dedent("""\
+          Configure the maximum retention period for on-demand backups.
+          The value must be greater than or equal to the minimum enforced
+          retention period set on the backup vault."""),
+  )
+
+
+def AddDiskBackupPlanProperties(parser):
+  """Adds the --disk-properties flag to the given parser."""
+  help_text = textwrap.dedent("""\
+      Workload-specific properties for disk backups generated by this plan.
+
+      *guest-flush*::: Indicates whether to perform a guest flush operation
+      before taking a disk backup. When set to true, the system will attempt
+      to ensure application-consistent backups. When set to false, the system
+      will create crash-consistent backups. Default value is false.
+
+      Example: --disk-properties=guest-flush=true
+      """)
+  parser.add_argument(
+      '--disk-properties',
+      metavar='PROPERTY=VALUE',
+      hidden=True,
+      type=arg_parsers.ArgDict(
+          spec={
+              'guest-flush': arg_parsers.ArgBoolean(),
+          },
+      ),
+      help=help_text,
+  )
